from .base import BasePaginator
from .inline import InlinePaginator
from .photo import PhotoPaginator
from .protocol import IPaginator
from .text import TextPaginator

__all__ = (
    "BasePaginator",
    "IPaginator",
    "InlinePaginator",
    "PhotoPaginator",
    "TextPaginator",
)
